%================================================================================
%
%               802.15.4 Channel Model
%
%================================================================================
%modified S-V channel model evaluation 
%Written by : Sun Xu, Kim Chee Wee, B. Kannan & Francois Chin on 22/02/2005 
%modified by: Ismail Lakkis with inputs from Shahriar Emami & Michael McLaughlin
%           : 1. Lines 54 to 62 From Shahriar/Michael
%           : 2. Modification of line 487 from Michael to accomodate antenna
%=================================================================================

function [g] = wbaLDRChan(cm_num,num_channels,DF,fs,f0)

%choose a sampling frequency close enough to 8GHz
OS  = round(8000/DF);
Fs  = OS*DF/1000;
MM  = round(fs/DF);

randn('state',12);    % initialize state of function for repeatability
rand('state',12);     % initialize state of function for repeatability

%get channel model params based on this channel model number
[Lam,Lmean,lambda_mode,lambda_1,lambda_2,beta,Gam,gamma_0,Kgamma, ...
 sigma_cluster,nlos,gamma_rise,gamma_1,chi,m0,Km,sigma_m0,sigma_Km, ...
 sfading_mode,m0_sp,std_shdw,kappa,fc,fs] = uwb_sv_params_15_4a( cm_num, Fs);


%get a realization (impulse responses)
ts = 1/fs;  % sampling frequency
[h_ct,t_ct,t0,np] = uwb_sv_model_ct_15_4a(Lam,Lmean,lambda_mode,lambda_1, ...
    lambda_2,beta,Gam,gamma_0,Kgamma,sigma_cluster,nlos,gamma_rise,gamma_1, ...
    chi,m0,Km,sigma_m0,sigma_Km,sfading_mode,m0_sp,std_shdw,num_channels,ts);

%change to complex baseband channel
h_ct_len = size(h_ct, 1);
phi = zeros(h_ct_len, 1);
for k = 1:num_channels
    phi = rand(h_ct_len, 1).*(2*pi);
    h_ct(:,k) = h_ct(:,k) .* exp(phi .* i);
end

% now reduce continuous-time result to a discrete-time result
[hN,N] = uwb_sv_cnvrt_ct_15_4a( h_ct, t_ct, np, num_channels, ts );

if N > 1,
  h = resample(hN, 1, N);  % decimate the columns of hN by factor N
else
  h = hN;
end

% add the frequency dependency
[h]= uwb_sv_freq_depend_ct_15_4a(h,fc,fs,num_channels,kappa);

%upconvert channel by approximately 2 GHz, LowPass Filter it & decimate it
LPF = wbaSrrcos(0.125,OS,8)';                               %or Os-1 to make the multipath bandwidth around 530MHz
Lh = length(h(:,1));
for k = 1:num_channels
    temp = h(:,k).*exp(j*2*pi*(6-f0*1e-3)*[0:Lh-1]'/Fs);    %upconvert by approximately 2GHz
    temp = conv(temp,LPF);                                  %Low pass filtering with filter of BW around 500 MHz 
    g(:,k) = resample(temp,MM,OS);                          %resample to go from Fs to fs
    g(:,k) = g(:,k)/sqrt(sum(abs(g(:,k).^2)));              %normalize energy to 1
end;

return;

%--------------------------------------------------------------------------
% Written by Sun Xu, Kim Chee Wee, B. Kannan & Francois Chin on 22/02/2004  
% Return modified S-V model parameters for standard UWB channel models
%--------------------------------------------------------------------------
% Lam    Cluster arrival rate (clusters per nsec)
% Lmean  Mean number of Clusters
% lambda_mode  Flag for Mixture of poission processes for ray arrival times
%              1 -> Mixture of poission processes for the ray arrival times
%              2 -> tapped delay line model
% lambda_1  Ray arrival rate for Mixture of poisson processes (rays per nsec)
% lambda_2  Ray arrival rate for Mixture of poisson processes (rays per nsec)
% beta      Mixture probability
%--------------------------------------------------------------------------
% Gam    Cluster decay factor (time constant, nsec)
% gamma0  Ray decay factor (time constant, nsec)
% Kgamma  Time dependence of ray decay factor
% sigma_cluster  Standard deviation of normally distributed variable for cluster energy
% nlos   Flag for non line of sight channel
%        0 -> LOS
%        1 -> NLOS with first arrival path starting at t ~= 0
%        2 -> NLOS with first arrival path starting at t = 0 and diffused first cluster
% gamma_rise  Ray decay factor of diffused first cluster (time constant, nsec)
% gamma_1     Ray decay factor of diffused first cluster (time constant, nsec)
% chi         Diffuse weight of diffused first cluster
%--------------------------------------------------------------------------
% m0     Mean of log-normal distributed nakagami-m factor
% Km     Time dependence of m0
% sigma_m0  Standard deviation of log-normal distributed nakagami-m factor
% sigma_Km  Time dependence of sigma_m0
% sfading_mode  Flag for small-scale fading
%               0 -> All paths have same m-factor distribution
%               1 -> LOS first path has a deterministic large m-factor
%               2 -> LOS first path of each cluster has a deterministic
%                    large m-factor
% m0_sp   Deterministic large m-factor
%--------------------------------------------------------------------------
% std_shdw  Standard deviation of log-normal shadowing of entire impulse response
%--------------------------------------------------------------------------
% kappa     Frequency dependency of the channel
%--------------------------------------------------------------------------
% fc        Center Frequency
% fs        Frequency Range
%
% modified by I2R
%--------------------------------------------------------------------------

function [Lam,Lmean,lambda_mode,lambda_1,lambda_2,beta,Gam,gamma_0,Kgamma, ...
        sigma_cluster,nlos,gamma_rise,gamma_1,chi,m0,Km,sigma_m0,sigma_Km, ...
        sfading_mode,m0_sp,std_shdw,kappa,fc,fs] = uwb_sv_params_15_4a( cm_num , Fs)

fs = Fs;
if cm_num == 1,        % Residential LOS
  % MPC arrival
  Lam = 0.047;   Lmean = 3;
  lambda_mode = 1; 
  lambda_1 = 1.54; lambda_2 = 0.15; beta = 0.095; 
  % MPC decay
  Gam = 22.61; gamma_0 =  12.53; Kgamma = 0; sigma_cluster = 2.75;
  nlos = 0; 
  gamma_rise = NaN; gamma_1 = NaN; chi = NaN; % dummy in this scenario
  % Small-scale Fading
  m0 = 0.67; Km = 0; sigma_m0 = 0.28; sigma_Km = 0;
  sfading_mode = 0; m0_sp = NaN;
  % Large-scale Fading -- Shadowing
  std_shdw = 2.22;
  % Frequency Dependence
  kappa = 1.12; 
  fc = 6;   % GHz
  %fs = 8;   % 2 - 10 GHz
  
elseif cm_num == 2,    % Residential NLOS
  % MPC arrival
  Lam = 0.12;   Lmean = 3.5;
  lambda_mode = 1; 
  lambda_1 = 1.77; lambda_2 = 0.15; beta = 0.045; 
  % MPC decay
  Gam = 26.27; gamma_0 =  17.5; Kgamma = 0; sigma_cluster = 2.93;
  nlos = 1; 
  gamma_rise = NaN; gamma_1 = NaN; chi = NaN; % dummy in this scenario
  % Small-scale Fading
  m0 = 0.69; Km = 0; sigma_m0 = 0.32; sigma_Km = 0;
  sfading_mode = 0; m0_sp = NaN;
  % Large-scale Fading -- Shadowing
  std_shdw = 3.51;
  % Frequency Dependence
  kappa = 1.53; 
  fc = 6;   % GHz
  %fs = 8;   % 2 - 10 GHz
  
elseif cm_num == 3,    % Office LOS
  % MPC arrival
  Lam = 0.016;   Lmean = 5.4;
  lambda_mode = 1; 
  lambda_1 = 0.19; lambda_2 = 2.97; beta = 0.0184; 
  % MPC decay
  Gam = 14.6; gamma_0 =  6.4; Kgamma = 0; sigma_cluster = 3; % assumption
  nlos = 0; 
  gamma_rise = NaN; gamma_1 = NaN; chi = NaN; % dummy in this scenario
  % Small-scale Fading
  m0 = 0.42; Km = 0; sigma_m0 = 0.31; sigma_Km = 0;
  sfading_mode = 2; m0_sp = 3; % assumption
  % Large-scale Fading -- Shadowing
  std_shdw = 0; %1.9;
  % Frequency Dependence
  kappa = 0.03;
  fc = 6;   % GHz
  %fs = 8;   % 3 - 6 GHz

elseif cm_num == 4,    % Office NLOS
  % MPC arrival
  Lam = 0.19;  Lmean = 3.1;
  lambda_mode = 1; 
  lambda_1 = 0.11; lambda_2 = 2.09; beta = 0.0096; 
  % MPC decay
  Gam = 19.8; gamma_0 =  11.2; Kgamma = 0; sigma_cluster = 3; % assumption
  nlos = 2; 
  gamma_rise = 15.21; gamma_1 = 11.84; chi = 0.78;
  % Small-scale Fading
  m0 = 0.5; Km = 0; sigma_m0 = 0.25; sigma_Km = 0;
  sfading_mode = 0; m0_sp = NaN; % assumption
  % Large-scale Fading -- Shadowing
  std_shdw = 3.9;
  % Frequency Dependence
  kappa =0.71;
  fc = 6;   % GHz
  %fs = 8;   % 3 - 6 GHz
  
elseif cm_num == 5,    % Outdoor LOS
  % MPC arrival
  Lam = 0.0448;  Lmean = 13.6;
  lambda_mode = 1; 
  lambda_1 = 0.13; lambda_2 = 2.41; beta = 0.0078;
  % MPC decay
  Gam = 31.7; gamma_0 =  3.7; Kgamma = 0; sigma_cluster = 3; % assumption
  nlos = 0; 
  gamma_rise = NaN; gamma_1 = NaN; chi = NaN; % dummy in this scenario
  % Small-scale Fading
  m0 = 0.77; Km = 0; sigma_m0 = 0.78; sigma_Km = 0;
  sfading_mode = 2; m0_sp = 3; % assumption
  % Large-scale Fading -- Shadowing
  std_shdw = 0.83;
  % Frequency Dependence
  kappa = 0.12;
  fc = 6;   % GHz
  %fs = 8;   % 3 - 6 GHz

elseif cm_num == 6,    % Outdoor NLOS
  % MPC arrival
  Lam = 0.0243;  Lmean = 10.5;
  lambda_mode = 1; 
  lambda_1 = 0.15; lambda_2 = 1.13; beta = 0.062;
  % MPC decay
  Gam = 104.7; gamma_0 =  9.3; Kgamma = 0; sigma_cluster = 3; % assumption
  nlos = 1; 
  gamma_rise = NaN; gamma_1 = NaN; chi = NaN; % dummy in this scenario
  % Small-scale Fading
  m0 = 0.56; Km = 0; sigma_m0 = 0.25; sigma_Km = 0;
  sfading_mode = 0; m0_sp = NaN; % assumption
  % Large-scale Fading -- Shadowing
  std_shdw = 2; % assumption
  % Frequency Dependence
  kappa = 0.13;
  fc =6;   % GHz
  %fs = 8;   % 3 - 6 GHz

elseif cm_num == 7,    % Industrial LOS
  % MPC arrival
  Lam = 0.0709;  Lmean = 4.75;
  lambda_mode = 2; 
  lambda_1 = 1; lambda_2 = 1; beta = 1; % dummy in this scenario
  % MPC decay
  Gam = 13.47; gamma_0 =  0.615; Kgamma = 0.926; sigma_cluster = 4.32;
  nlos = 0; 
  gamma_rise = NaN; gamma_1 = NaN; chi = NaN; % dummy in this scenario
  % Small-scale Fading
  m0 = 0.36; Km = 0; sigma_m0 = 1.13; sigma_Km = 0;
  sfading_mode = 1; m0_sp = 12.99;
  % Large-scale Fading -- Shadowing
  std_shdw = 6;
  % Frequency Dependence
  kappa = -1.103;   
  fc = 6;   % GHz
  %fs = 8;   % 2 - 8 GHz
  
elseif cm_num == 8,    % Industrial NLOS
  % MPC arrival
  Lam = 0.089;  Lmean = 1; 
  lambda_mode = 2; 
  lambda_1 = 1; lambda_2 = 1; beta = 1; % dummy in this scenario
  % MPC decay
  Gam = 5.83; gamma_0 =  0.3; Kgamma = 0.44; sigma_cluster = 2.88;
  nlos = 2; 
  gamma_rise = 47.23; gamma_1 = 84.15; chi = 0.99; 
  % Small-scale Fading
  m0 = 0.3; Km = 0; sigma_m0 = 1.15; sigma_Km = 0;
  sfading_mode = 0; m0_sp = NaN; % m0_sp is assumption
  % Large-scale Fading -- Shadowing
  std_shdw = 6;
  % Frequency Dependence
  kappa = -1.427;   
  fc = 6;   % GHz
  %fs = 8;   % 2 - 8 GHz
  
elseif cm_num == 9,    % Open Outdoor Environment NLOS (Fram, Snow-Covered Open Area)
  % MPC arrival
  Lam = 0.0305;  Lmean = 3.31;
  lambda_mode = 1; 
  lambda_1 = 0.0225; lambda_2 = 1; beta = 1;
  % MPC decay
  Gam = 56; gamma_0 =  0.92; Kgamma = 0; sigma_cluster = 3; % sigma_cluster is assumption
  nlos = 1; 
  gamma_rise = NaN; gamma_1 = NaN; chi = NaN; 
  % Small-scale Fading
  m0 = 4.1; Km = 0; sigma_m0 = 2.5; sigma_Km = 0;
  sfading_mode = 0; m0_sp = NaN; % m0_sp is assumption
  % Large-scale Fading -- Shadowing
  std_shdw = 3.96;
  % Frequency Dependence
  kappa = -1;   % Kappa is assumption
  fc = 6;   % GHz
  %fs = 8;   % 2 - 8 GHz

else
  error('cm_num is wrong!!')
end

return

%--------------------------------------------------------------------------
% Written by Sun Xu, Kim Chee Wee, B. Kannan & Francois Chin on 22/02/2005
%   IEEE 802.15.4a UWB channel model for PHY proposal evaluation
%      continuous-time realization of modified S-V channel model
% Input parameters:
%   detailed introduction of input parameters is at uwb_sv_params.m
%   num_channels  number of random realizations to generate
% Outputs
%   h is returned as a matrix with num_channels columns, each column 
%     holding a random realization of the channel model (an impulse response)
%   t is organized as h, but holds the time instances (in nsec) of the paths whose
%     signed amplitudes are stored in h
%   t0 is the arrival time of the first cluster for each realization
%   np is the number of paths for each realization.
% Thus, the k'th realization of the channel impulse response is the sequence 
%   of (time,value) pairs given by (t(1:np(k),k), h(1:np(k),k))
%
% modified by I2R
%--------------------------------------------------------------------------
function [h,t,t0,np] = uwb_sv_model_ct_15_4a(Lam,Lmean,lambda_mode,lambda_1, ...
    lambda_2,beta,Gam,gamma_0,Kgamma,sigma_cluster,nlos,gamma_rise,gamma_1, ...
    chi,m0,Km,sigma_m0,sigma_Km,sfading_mode,m0_sp,std_shdw,num_channels,ts)

% initialize and precompute some things
std_L = 1/sqrt(2*Lam);      % std dev (nsec) of cluster arrival spacing
std_lam_1 = 1/sqrt(2*lambda_1);
std_lam_2 = 1/sqrt(2*lambda_2);
% std_lam = 1/sqrt(2*lambda); % std dev (nsec) of ray arrival spacing
h_len = 1000;  % there must be a better estimate of # of paths than this
ngrow = 1000; % amount to grow data structure if more paths are needed
h = zeros(h_len,num_channels);
t = zeros(h_len,num_channels);
t0 = zeros(1,num_channels);
np = zeros(1,num_channels);

for k = 1:num_channels       % loop over number of channels
  tmp_h = zeros(size(h,1),1);
  tmp_t = zeros(size(h,1),1);
  if nlos == 1,
    Tc = (std_L*randn)^2 + (std_L*randn)^2;  % First cluster random arrival
  else
    Tc = 0;                   % First cluster arrival occurs at time 0
  end
  t0(k) = Tc;
  
  if nlos == 2 & lambda_mode == 2
      L = 1;  % for industrial NLOS environment
  else
      L = max(1, poissrnd(Lmean));       % number of clusters
  end

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if Kgamma ~= 0 & nlos == 0
    Tcval = [];  Tc_cluster= [];
    Tc_cluster(1,1)=Tc;
    for i_Tc=2:L+1
      Tc_cluster(1,i_Tc)= Tc_cluster(1,i_Tc-1)+(std_L*randn)^2 + (std_L*randn)^2;
    end
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  cluster_index = zeros(1,L);
  path_ix = 0;
  nak_m = [];
  for ncluster = 1:L
    % Determine Ray arrivals for each cluster
    Tr = 0;  % first ray arrival defined to be time 0 relative to cluster
    cluster_index(ncluster) = path_ix+1;  % remember the cluster location
    gamma = Kgamma*Tc + gamma_0; % delay dependent cluster decay time
    if nlos == 2 & ncluster == 1
      gamma = gamma_1;
    end
    Mcluster = sigma_cluster*randn;
    Pcluster = 10*log10(exp(-1*Tc/Gam))+Mcluster; % total cluster power
    Pcluster = 10^(Pcluster*0.1);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if Kgamma ~= 0 & nlos == 0
      Tr_len=Tc_cluster(1,ncluster+1)-Tc_cluster(1,ncluster);
    else
      Tr_len = 10*gamma;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    while (Tr < Tr_len),
      t_val = (Tc+Tr);  % time of arrival of this ray
      if nlos == 2 & ncluster == 1
        % equation (22)
        h_val = Pcluster*(1-chi*exp(-Tr/gamma_rise))*exp(-Tr/gamma_1) ...
               *(gamma+gamma_rise)/gamma/(gamma+gamma_rise*(1-chi));
      else
        % equation (19)
        h_val = Pcluster/gamma*exp(-Tr/gamma)/(beta*lambda_1+(1-beta)*lambda_2+1);
      end
      path_ix = path_ix + 1;  % row index of this ray
      if path_ix > h_len,
        % grow the output structures to handle more paths as needed
        tmp_h = [tmp_h; zeros(ngrow,1)];
        tmp_t = [tmp_t; zeros(ngrow,1)];
        h = [h; zeros(ngrow,num_channels)];
        t = [t; zeros(ngrow,num_channels)];
        h_len = h_len + ngrow;
      end
      tmp_h(path_ix) = h_val;
      tmp_t(path_ix) = t_val;
%       if lambda_mode == 0
%         Tr = Tr + (std_lam*randn)^2 + (std_lam*randn)^2;
      if lambda_mode == 1
        if rand < beta
          Tr = Tr + (std_lam_1*randn)^2 + (std_lam_1*randn)^2;
        else
          Tr = Tr + (std_lam_2*randn)^2 + (std_lam_2*randn)^2;
        end
      elseif lambda_mode == 2
        Tr = Tr + ts;
      else
        error('lambda mode is wrong!')
      end
      % generate log-normal distributed nakagami m-factor
      m_mu = m0 - Km*t_val;
      m_std = sigma_m0 - sigma_Km*t_val;
      nak_m = [nak_m, lognrnd(m_mu, m_std)];
    end
    Tc = Tc + (std_L*randn)^2 + (std_L*randn)^2;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if Kgamma ~= 0 & nlos == 0
       Tc = Tc_cluster(1,ncluster+1);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  end

  % change m value of the first multipath to be the deterministic value
  if sfading_mode == 1
    nak_ms(cluster_index(1)) = m0_sp;
  elseif sfading_mode == 2
    nak_ms(cluster_index) = m0_sp;
  end
  
  % apply nakagami
  for path = 1:path_ix
    h_val = (gamrnd(nak_m(path), tmp_h(path)/nak_m(path))).^(1/2);
    tmp_h(path) = h_val;
  end

  np(k) = path_ix;  % number of rays (or paths) for this realization
  [sort_tmp_t,sort_ix] = sort(tmp_t(1:np(k)));    % sort in ascending time order
  t(1:np(k),k) = sort_tmp_t;
  h(1:np(k),k) = tmp_h(sort_ix(1:np(k)));
  % now impose a log-normal shadowing on this realization
%   fac = 10^(std_shdw*randn/20) / sqrt( h(1:np(k),k)' * h(1:np(k),k) );
%   h(1:np(k),k) = h(1:np(k),k) * fac;
end
return


%-------------------------------------------------------------------------------------------
% convert continuous-time channel model h_ct to N-times oversampled discrete-time samples
% h_ct, t, np, and num_channels are as specified in uwb_sv_model
% ts is the desired time resolution
% 
% hN will be produced with time resolution ts / N.
% It is up to the user to then apply any filtering and/or complex downconversion and then
% decimate by N to finally obtain an impulse response at time resolution ts.
%-------------------------------------------------------------------------------------------
function [hN,N] = uwb_sv_cnvrt_ct_15_4a( h_ct, t, np, num_channels, ts )

min_Nfs = 100; % GHz
N = max( 1, ceil(min_Nfs*ts) );  % N*fs = N/ts is the intermediate sampling frequency before decimation
N = 2^nextpow2(N); % make N a power of 2 to facilitate efficient multi-stage decimation

Nfs = N / ts;
t_max = max(t(:));  % maximum time value across all channels
h_len = 1 + floor(t_max * Nfs);  % number of time samples at resolution ts / N
hN = zeros(h_len,num_channels);
for k = 1:num_channels
  np_k = np(k);  % number of paths in this channel
  t_Nfs = 1 + floor(t(1:np_k,k) * Nfs);  % vector of quantized time indices for this channel
  for n = 1:np_k
    hN(t_Nfs(n),k) = hN(t_Nfs(n),k) + h_ct(n,k);
  end
end
return

%------------------------------------------------------------
% This function is used to include the frequency dependency 
%------------------------------------------------------------
function [h]= uwb_sv_freq_depend_ct_15_4a(h,fc,fs,num_channels,kappa)

f0 = 5; % GHz
h_len = length(h(:,1));
f = [fc-fs/2 : fs/h_len/2 : fc+fs/2]./f0;
f = f.^(-2*(kappa+1));  %kappa+1 instead of kapp to account for the antenna as per Michael recommendation
f = [f(h_len : 2*h_len), f(1 : h_len-1)]';
i = (-1)^(1/2);    % complex i
for  c = 1:num_channels
    % add the frequency dependency
    h2 = zeros(2*h_len, 1);
    h2(1 : h_len) = h(:,c);  % zero padding
    fh2 = fft(h2);
    fh2 = fh2 .* f;
    h2 = ifft(fh2);
    h(:,c) = h2(1:h_len);
    
    % Normalize the channel energy to 1
    h(:,c) = h(:,c)/sqrt(h(:,c)' * h(:,c) );
end
return
